@@ -10,49 +10,50 @@
 #'   `force_download`, and `max_file_size` are ignored if `file_path` points to
 #'   an existing `.pbf` or `.gpkg` file.
 #'
-#'   You cannot add any field or layer to an existing `.gpkg` file (unless you
-#'   have the `.pbf` file and you convert it again with a different
-#'   configuration), but you can extract some of the tags in `other_tags` field.
-#'   Check examples and [oe_get_keys()] for more details.
+#'   Please note that you cannot add any field to an existing `.gpkg` file using
+#'   the argument `extra_tags` without rerunning the vectortranslate process on
+#'   the corresponding `.pbf` file. On the other hand, you can extract some of
+#'   the tags in `other_tags` field as new columns. See examples and
+#'   [oe_get_keys()] for more details.
 #'
 #' @inheritParams oe_get
-#' @param file_path A URL or the path of a `.pbf` or `.gpkg` file. If a URL,
+#' @param file_path A URL or the path to a `.pbf` or `.gpkg` file. If a URL,
 #'   then it must be specified using HTTP/HTTPS protocol.
 #' @param file_size How big is the file? Optional. `NA` by default. If it's
 #'   bigger than `max_file_size` and the function is run in interactive mode,
 #'   then an interactive menu is displayed, asking for permission to download
 #'   the file.
 #'
-#' @return An `sf` object.
+#' @return An `sf` object or, when `download_only` argument equals `TRUE`, a
+#'   character vector.
 #' @export
 #'
 #' @examples
 #' # Read an existing .pbf file. First we need to copy a .pbf file into a
 #' # temporary directory
+#' its_pbf = file.path(tempdir(), "test_its-example.osm.pbf")
 #' file.copy(
 #'   from = system.file("its-example.osm.pbf", package = "osmextract"),
-#'   to = file.path(tempdir(), "its-example.osm.pbf")
+#'   to = its_pbf
 #' )
-#'
-#' my_pbf = file.path(tempdir(), "its-example.osm.pbf")
-#' oe_read(my_pbf)
+#' oe_read(its_pbf)
 #'
 #' # Read a new layer
-#' oe_read(my_pbf, layer = "points")
+#' oe_read(its_pbf, layer = "points")
 #'
 #' # The following example shows how to add new tags
-#' names(oe_read(my_pbf, extra_tags = c("oneway", "ref"), quiet = TRUE))
+#' names(oe_read(its_pbf, extra_tags = c("oneway", "ref"), quiet = TRUE))
 #'
-#' # Read an existing .gpkg file. This file was created by oe_read
-#' my_gpkg = file.path(tempdir(), "its-example.gpkg")
-#' oe_read(my_gpkg)
+#' # Read an existing .gpkg file. This file was created internally by oe_read().
+#' its_gpkg = file.path(tempdir(), "test_its-example.gpkg")
+#' oe_read(its_gpkg)
 #'
-#' # You cannot add any layer to an existing .gpkg file but you can extract some
-#' # of the tags in other_tags. Check oe_get_keys() for more details.
-#' names(oe_read(my_gpkg, extra_tags = c("maxspeed"))) # doesn't work
+#' # You cannot add any new layer to an existing .gpkg file but you can extract
+#' # some of the tags in other_tags. Check oe_get_keys() for more details.
+#' names(oe_read(its_gpkg, extra_tags = c("maxspeed"))) # doesn't work
 #' # Instead, use the query argument
 #' names(oe_read(
-#'   my_gpkg,
+#'   its_gpkg,
 #'   quiet = TRUE,
 #'   query =
 #'   "SELECT *,
@@ -69,8 +70,7 @@
 #' oe_read(my_url, provider = "test", quiet = FALSE)}
 #'
 #' # Remove .pbf and .gpkg files in tempdir
-#' # (since they may interact with other examples)
-#' file.remove(list.files(path = tempdir(), pattern = "(pbf|gpkg)", full.names = TRUE))
+#' oe_clean(tempdir())
 oe_read = function(
   file_path,
   layer = "lines",
@@ -95,15 +95,32 @@ oe_read = function(
   # Test misspelt arguments
   check_layer_provider(layer, provider)
 
-  # Test if there is misalignment between query and layer. See also
-  # See https://github.com/ropensci/osmextract/issues/122
-  if ("query" %in% names(list(...))) {
-    # Check if the query argument defined in sf::st_read was defined using a
-    # layer different than layer argument.
+  # Check that all arguments inside ... are named arguments. See also
+  # https://github.com/ropensci/osmextract/issues/234. I extract the names in
+  # ... and save the result in dots_names since names(list(...)) returns an
+  # error when there is a missing element in ... See the examples in utils.R. I
+  # need to check null and "" values. See utils.R for examples. See
+  # https://github.com/ropensci/osmextract/issues/241 and corresponding PR for a
+  # discussion.
+  dots_names = extract_dots_names_safely(...)
+  if (...length() && (any(is.null(dots_names)) | any(dots_names == ""))) {
+    oe_stop(
+      .subclass = "oe_read-namesDotsError",
+      message = "All arguments in oe_get() and oe_read() beside 'place' and 'layer' must be named. Please check also that you didn't add an extra comma at the end of your call.",
+    )
+  }
+
+  # Test if there is a misalignment between query and layer argument. See also
+  # https://github.com/ropensci/osmextract/issues/122. Moreover, I had to use
+  # ...names() instead of names(list(...)) because of
+  # https://github.com/ropensci/osmextract/issues/234
+  if ("query" %in% dots_names) {
+    # Check if the query argument (which is passed to sf::st_read) was defined using a
+    # layer different than layer argument. Indeed:
     # Extracted from sf::st_read docs: For query with a character dsn the query
     # text is handed to 'ExecuteSQL' on the GDAL/OGR data set and will result in
-    # the creation of a new layer (and layer is ignored) See also
-    # https://github.com/ropensci/osmextract/issues/122
+    # the creation of a new layer (and layer is ignored).
+    # See also https://github.com/ropensci/osmextract/issues/122
     query = list(...)[["query"]]
 
     # Extract everything that is specified after FROM or from
@@ -111,10 +128,12 @@ oe_read = function(
     layer_raw = regmatches(query, regexpr(query_pattern, query, perl = TRUE))
 
     if (length(layer_raw) != 1L) {
-      stop(
-        "There is an error in the query. Please open a new issue at ",
-        "https://github.com/ropensci/osmextract/issues",
-        call. = FALSE
+      oe_stop(
+        .subclass = "oe_read-errorInQuery1",
+        message = paste0(
+          "There is an error in the query or in oe_read. Please open a new issue at ",
+          "https://github.com/ropensci/osmextract/issues"
+        )
       )
     }
 
@@ -125,19 +144,16 @@ oe_read = function(
     )
 
     if (length(layer_clean) != 1L) {
-      stop(
-        "There is an error in the query. Please open a new issue at ",
-        "https://github.com/ropensci/osmextract/issues",
-        call. = FALSE
+      oe_stop(
+        .subclass = "oe_read-errorInQuery2",
+        message = paste0(
+          "There is an error in the query or in oe_read. Please open a new issue at ",
+          "https://github.com/ropensci/osmextract/issues"
+        )
       )
     }
 
     if (layer_clean[[1]] != layer) {
-      warning(
-        "The query selected a layer which is different from layer argument. ",
-        "We will replace the layer argument.",
-        call. = FALSE
-      )
       layer = layer_clean[[1]]
     }
   }
@@ -148,15 +164,15 @@ oe_read = function(
     # arguments using ...
     ...length() > 0L &&
     # At the moment, the only function that uses ... is sf::st_read, so I can
-    # simply check which arguments in ... are not inlucded in the formals of
+    # simply check which arguments in ... are not included in the formals of
     # sf::st_read. Moreover, st_read should always use sf:::st_read.character,
     # so I need to check the formals of that method.
-    #
+
     # The classical way should be names(formals(sf:::st_read.character)), but it
     # returns a notes in the R CMD checks related to :::. Hence, I decided to
     # use names(formals("st_read.character", envir = getNamespace("sf")))
     any(
-      names(list(...)) %!in%
+      dots_names %!in%
       # The ... arguments in st_read are passed to st_as_sf so I need to add the
       # formals of st_as_sf.
       # See https://github.com/ropensci/osmextract/issues/152
@@ -168,10 +184,10 @@ oe_read = function(
     )
   ) {
     warning(
-      "The following arguments are probably misspelled: ",
+      "The following arguments were probably misspelled: ",
       paste(
         setdiff(
-          names(list(...)),
+          dots_names,
           union(
             names(formals(get("st_read.character", envir = getNamespace("sf")))),
             names(formals(get("st_as_sf.data.frame", envir = getNamespace("sf"))))
@@ -179,55 +195,26 @@ oe_read = function(
         ),
         collapse = " - "
       ),
-      call. = FALSE
+      call. = FALSE,
+      immediate. = TRUE
     )
   }
 
 
-  # If the input file_path is an existing .gpkg file, then this is the easiest
-  # case since we only need to read it:
+  # If the input file_path points to an existing .gpkg file, then this is the
+  # easiest case since we only need to read it:
   if (file.exists(file_path) && tools::file_ext(file_path) == "gpkg") {
-    # I need the following if to return the .gpkg file path in oe_get
     if (isTRUE(download_only)) {
       return(file_path)
     }
 
-    # Starting from sf 1.0.2, sf::st_read raises a warning message when both
-    # layer and query arguments are set (while it raises a warning in sf < 1.0.2
-    # when there are multiple layers and the layer argument is not set). See
-    # also https://github.com/r-spatial/sf/issues/1444
-    if (utils::packageVersion("sf") <= "1.0.1") {
-      return(sf::st_read(
-        dsn = file_path,
-        layer = layer,
-        quiet = quiet,
-        ...
-      ))
-    } else {
-      if ("query" %in% names(list(...))) {
-        return(sf::st_read(
-          dsn = file_path,
-          quiet = quiet,
-          ...
-        ))
-      } else {
-        return(sf::st_read(
-          dsn = file_path,
-          layer = layer,
-          quiet = quiet,
-          ...
-        ))
-      }
-    }
-
-    return(sf::st_read(file_path, layer, quiet = quiet, ...))
+    return(my_st_read(dsn = file_path, layer = layer, quiet = quiet, ...))
   }
 
-  # Now I think I can assume that file_path is a URL or points to a .pbf file. I
-  # assume that if file.exists(file_path) is FALSE then file_path is a URL and I
-  # need to download the file
+  # Now I think I can assume that file_path represents a URL or points to a .pbf
+  # file. I assume that if file.exists(file_path) is FALSE then file_path is a
+  # URL and I need to download the file
   if (!file.exists(file_path)) {
-
     # Add an if clause to check if file_path "looks like" a URL
     # See https://github.com/ropensci/osmextract/issues/134 and
     # https://stackoverflow.com/questions/3809401/what-is-a-good-regular-expression-to-match-a-url.
@@ -237,10 +224,12 @@ oe_read = function(
     like_url = is_like_url(file_path)
 
     if (!like_url) {
-      stop(
-        "The input file_path does not correspond to any existing file ",
-        "and it doesn't look like a URL.",
-        call. = FALSE
+      oe_stop(
+        .subclass = "oe_read-inputDoesNotCorrespondToExistingFileOrURL",
+        message = paste0(
+          "The input file_path does not correspond to any existing file ",
+          "and it doesn't look like a URL."
+        )
       )
     }
 
@@ -255,34 +244,43 @@ oe_read = function(
     )
   }
 
-  # Now file_path should always be the path to an existing .pbf or .gpkg file
-  # Again, if file_path points to an existing .gpkg file:
-  if (file.exists(file_path) && tools::file_ext(file_path) == "gpkg") {
+  if (!file.exists(file_path)) {
+    oe_stop(
+      .subclass = "oe_read-errorAfterDownload",
+      message = paste0(
+        "An error occurred during the download process. ",
+        "Please raise a new issue at https://github.com/ropensci/osmextract/issues"
+      )
+    )
+  }
+
+  # Now file_path should always point to an existing .pbf or .gpkg file Again,
+  # if file_path points to an existing .gpkg file:
+  if (tools::file_ext(file_path) == "gpkg") {
     if (isTRUE(download_only)) {
       return(file_path)
     }
 
-    return(sf::st_read(file_path, layer, quiet = quiet, ...))
+    return(my_st_read(dsn = file_path, layer = layer, quiet = quiet, ...))
   }
 
   # Now file_path should always point to an existing .pbf file. If the user set
   # skip_vectortranslate = TRUE, then we just need to return the pbf path or
   # read it.
   if (
-    file.exists(file_path) &&
-    tools::file_ext(file_path) == "pbf" &&
+    tools::file_ext(file_path) %in% c("pbf", "osm") &&
     isTRUE(skip_vectortranslate)
   ) {
     if (isTRUE(download_only)) {
       return(file_path)
     }
 
-    return(sf::st_read(file_path, layer, quiet = quiet, ...))
+    return(my_st_read(dsn = file_path, layer = layer, quiet = quiet, ...))
   }
 
-  # See https://github.com/ropensci/osmextract/issues/144. The vectortranslate
-  # operation should never be skipped if the user forced the download of a new
-  # .osm.pbf file.
+  # The vectortranslate operation should never be skipped if the user forced the
+  # download of a new .osm.pbf file. See
+  # https://github.com/ropensci/osmextract/issues/144.
   if (isTRUE(force_download)) {
     never_skip_vectortranslate = TRUE
     force_vectortranslate = TRUE
@@ -304,44 +302,19 @@ oe_read = function(
     quiet = quiet
   )
 
-  # This is just for returning the .gpkg file path in case I need it for
-  # something
+  # Return the .gpkg file path
   if (isTRUE(download_only)) {
     return(gpkg_file_path)
   }
 
   # Add another test since maybe there was an error during the vectortranslate process:
   if (!file.exists(gpkg_file_path)) {
-    stop("An error occurred during the vectortranslate process", call. = FALSE)
-  }
-
-  # Read the translated file with sf::st_read. Moreover, starting from sf 1.0.2,
-  # sf::st_read raises a warning message when both layer and query arguments are
-  # set (while it raises a warning in sf < 1.0.2 when there are multiple layers
-  # and the layer argument is not set). See also
-  # https://github.com/r-spatial/sf/issues/1444
-
-  if (utils::packageVersion("sf") <= "1.0.1") {
-    sf::st_read(
-      dsn = gpkg_file_path,
-      layer = layer,
-      quiet = quiet,
-      ...
+    oe_stop(
+      .subclass = "oe_read-errorAfterVectortranslate",
+      message = "A fatal error occurred during the vectortranslate process."
     )
-  } else {
-    if ("query" %in% names(list(...))) {
-      sf::st_read(
-        dsn = gpkg_file_path,
-        quiet = quiet,
-        ...
-      )
-    } else {
-      sf::st_read(
-        dsn = gpkg_file_path,
-        layer = layer,
-        quiet = quiet,
-        ...
-      )
-    }
   }
+
+  # Read the file
+  my_st_read(gpkg_file_path, layer = layer, quiet = quiet, ...)
 }
